@@ -71,7 +71,7 @@ class XNetOptionParser(optparse.OptionParser):
         self.add_option('-i', '--interval', dest='interval', metavar='nsecs',
                         help='time interval between each action ')
         self.add_option('', '--repeat', dest='repeat', metavar='ntimes',
-                        help='repeat action this many times (0 = infinity)')
+                        help='repeat action this many times (-1 = infinity)')
         self.add_option('-T', '--supress-timeout', dest='supress_timeout',
                         help='supress messages from tasks timing out', action='store_true')
         self.add_option('-f', '--format', dest='format', metavar='fmt',
