@@ -10,9 +10,12 @@ __all__ = ['WebgetTool']
 import unittest
 import doctest
 import sys
+import os
 import urllib2
+import codecs
 
 from xnet.tools import Tool
+#from xnet.debug import pdb
 
 
 class WebgetException(Exception):
@@ -36,6 +39,12 @@ class WebgetTool(Tool):
             dict(dest='proxy')),
         ('', '--code', 'only print response code',
             dict(dest='code', action='store_true')),
+        ('', '--xpath', 'grab elements matching xpath expression',
+            dict(action='append', metavar='expr')),
+        ('', '--xpath-delimiter', 'xpath output delimiter when using multiple --xpath\'s',
+            dict(dest='xpath_delimiter', action='store_true')),
+        ('', '--xpath-strict', 'exit if not all --xpath\'s are matched',
+            dict(dest='xpath_strict', action='store_true')),
     ]
 
     @classmethod
@@ -72,6 +81,7 @@ class WebgetTool(Tool):
         _host = parse_result['host']
         _url = parse_result['url']
         _response = None
+        _body_unicode = None
         _code = None
         _msg = None
         _body = None
@@ -97,7 +107,9 @@ class WebgetTool(Tool):
                 raise WebgetException(errmsg)
             #request.set_proxy(self.options.proxy, 'https')
 
-        read_response = not self.options.code and not self.options.server
+        read_response = not self.options.code \
+                        and not self.options.server
+
         if self.options.format and '{body}' in self.options.format:
             read_response = True
 
@@ -116,7 +128,10 @@ class WebgetTool(Tool):
                 _body = e.read()
         except urllib2.URLError, e:
             errmsg = _url + ' '
-            errmsg += e.reason.strerror.replace(' ', '-').replace(',', '-')
+            if type(e.reason) is str:
+                errmsg += e.reason.replace(' ', '-').replace(',', '-')
+            else:
+                errmsg += e.reason.strerror.replace(' ', '-').replace(',', '-')
             self.set_error(errmsg)
             self.stderr = True
             return None
@@ -130,20 +145,38 @@ class WebgetTool(Tool):
                 _response.close()
             except:
                 pass
+        if not _response:
+            return None
+        content_type = _response.headers.get('content-type', None)
+        charset = None
+        if content_type:
+            w = content_type.split(';')
+            content_type = w[0]
+            for s in w[1:]:
+                s = s.strip()
+                if s.startswith('charset='):
+                    charset = s.split('charset=', 1)[1]
+        if charset:
+            _body_unicode = codecs.decode(_body, charset)
+        else:
+            _body_unicode = codecs.decode(_body, 'Latin-1')
+            #_body_unicode = unicode(_body)
+            
         result['host'] = _host
         result['url'] = _url
         result['response'] = _response
         result['code'] = _code
         result['msg'] = _msg
         result['body'] = _body
+        result['body_unicode'] = _body_unicode
         return result
 
-    def __format__(self, line, parse_result, value):
+    def __format__(self, line, parse_result, action_result):
         options = self.options
-        url = value['url']
-        response = value['response']
-        code = value['code']
-        msg = value['msg']
+        url = action_result['url']
+        response = action_result['response']
+        code = action_result['code']
+        msg = action_result['msg']
         output = ''
         #
         if options.server:
@@ -154,8 +187,47 @@ class WebgetTool(Tool):
                 output = '{0} {1} {2}'.format(url, code, msg)
             else:
                 output = '{0} {1}'.format(url, code)
+        elif options.xpath:
+            try:
+                import lxml.etree
+            except ImportError:
+                import time
+                errmsg = '\n'
+                errmsg += '*** XNET USAGE INFORMATION ***\n'
+                errmsg += '\n'
+                errmsg += 'ImportError raised, lxml.etree might be missing.\n'
+                errmsg += 'For Debian, try:\n'
+                errmsg += '  $ sudo apt-get install python-lxml\n'
+                errmsg += 'For OSX, try:\n'
+                errmsg += '  $ sudo port install py-lxml\n'
+                print errmsg
+                time.sleep(3)
+                raise
+            htmlparser = lxml.etree.HTMLParser()
+            tree = lxml.etree.fromstring(action_result['body_unicode'], htmlparser)
+            for xpath in self.options.xpath:
+                xpath_result = tree.xpath(xpath)
+                success = False
+                for elem in xpath_result:
+                    success = True 
+                    if type(elem) is lxml.etree._ElementStringResult:
+                        elem = str(elem)
+                    if not type(elem) is str:
+                        elem = lxml.etree.tostring(elem)
+                    output += elem
+                    if len(output) and output[-1] != '\n':
+                        output += '\n'
+                if self.options.xpath_strict and not success:
+                    import tempfile
+                    (fd, name) = tempfile.mkstemp(prefix='webget-xpath-fail-')
+                    os.write(fd, action_result['body'])
+                    os.close(fd)
+                    msg = 'Error: XPath not matched: {0}'.format(xpath)
+                    msg += ', body saved to {0}'.format(name)
+                    sys.exit(msg)
+                output += self.options.xpath_delimiter or ' '
         else:
-            output = value['body']
+            output = action_result['body']
         return output
 
     @classmethod
